class Base:
    size = 6
    debug = False
    font_size = 50
    timeout = 6000
    type = "str-int"
    image_height = 70
    image_weight = 180
    text_color= "random"
    background_color= "#fff"


def init(**kwargs):
    for key, value in kwargs.items():
        if key == "timeout":
            Base.timeout = value

        if key == "size" and (value > 2 or value < 10):
            Base.size = value

        if key == "debug" and (value == False or value == True):
            Base.debug = value

        if key == "type" and (value == "int" or value == "str"):
            Base.type = value



try:
    import redjlog
    from .management.setting import Base
    from .models import RedjCaptchaModel

    def fullCheckCaptcha(request):
        if Base.debug == True:
            return True
        key = None
        value = None
        if request.method == 'GET':
            key = request.GET.get('captcha_key')
            value = request.GET.get('captcha_value')
        elif request.method == 'POST':
            key = request.POST.get('captcha_key')
            value = request.POST.get('captcha_value')

        if key == None or value == None:
            raise redjlog.OtherException("Captcha inValid")
        checkModel= RedjCaptchaModel.checkCaptcha(key, value, request)
        if checkModel:
            return True
        raise redjlog.OtherException("Captcha inValid")
except:
    pass


import uuid
from django.db import models
from datetime import timedelta
from django.utils import timezone
from .management.setting import Base
from .management.helper import randomString


class RedjCaptchaModel(models.Model):
    id = models.UUIDField(
        unique=True,
        editable=False,
        primary_key=True,
        default=uuid.uuid4,
    )
    text = models.CharField(max_length=12)
    request_ip = models.CharField(null=True, max_length=40)
    user_agent = models.CharField(null=True, max_length=250)
    created_by = models.UUIDField(null=True, editable=False)
    request_host = models.CharField(null=True, max_length=250)
    deleted_at = models.DateTimeField(null=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)
    created_at = models.DateTimeField(auto_now_add=True, editable=False)

    class Meta:
        ordering = ['-created_at']
        db_table = "redj_captchas"

    def __str__(self):
        return str(self.id)

    def create(request):
        set_int = True
        set_str = True
        if Base.type == "str":
            set_str = True
            set_int = False
        if Base.type == "int":
            set_int = True
            set_str = False

        user_agent = request.headers.get('User-Agent', "empty")
        text = randomString(Base.size, set_int=set_int, set_str=set_str)
        model = RedjCaptchaModel.objects.create(
            text=text,
            user_agent=user_agent,
            request_ip=request.META['REMOTE_ADDR'],
            request_host=request.META['HTTP_HOST']
        )
        return model

    def checkCaptcha(key, value, request=None):
        try:
            expier_at = timezone.now() - timedelta(seconds=int(Base.timeout))
            filter = {
                "id": key,
                "text": value,
                "deleted_at": None,
                "created_at__gte": expier_at
            }
            if request != None:
                user_agent = request.headers.get('User-Agent', "empty")
                filter["user_agent"] = user_agent
                filter["request_ip"] = request.META['REMOTE_ADDR']
                filter["request_host"] = request.META['HTTP_HOST']
            model = RedjCaptchaModel.objects.filter(**filter)
            if len(model) > 0:
                RedjCaptchaModel.objects.filter(
                    **filter).update(deleted_at=timezone.now())
                return True
        except:
            pass

        return False



from .models import RedjCaptchaModel
from .management.setting import Base
from django.http import JsonResponse
from .management.helper import generateImage


def getCaptcha(request):
    model = RedjCaptchaModel.create(request)
    image = generateImage(model.text)
    return JsonResponse({
        "status": 200,
        "data": {
            "key": model.id,
            "timeout": Base.timeout,
            "created_at": model.created_at,
            "image": image,
        }
    })


def checkCaptcha(key, value, request=None):
    if Base.debug == True:
        return True
    return RedjCaptchaModel.checkCaptcha(key, value, request)

def fullCheckCaptcha(request):
    if Base.debug == True:
        return True
    key=None
    value=None
    if request.method == 'GET':
        key=request.GET.get('captcha_key')
        value=request.GET.get('captcha_value')
    elif request.method == 'POST':
        key=request.POST.get('captcha_key')
        value=request.POST.get('captcha_value')

    if key==None or value==None:
        return  False
    return RedjCaptchaModel.checkCaptcha(key, value, request)



# Generated by Django 4.0 on 2022-01-04 07:06

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RedjCaptchaModel',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True)),
                ('text', models.CharField(max_length=12)),
                ('request_ip', models.CharField(max_length=40, null=True)),
                ('user_agent', models.CharField(max_length=250, null=True)),
                ('created_by', models.UUIDField(editable=False, null=True)),
                ('request_host', models.CharField(max_length=250, null=True)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'db_table': 'redj_captchas',
                'ordering': ['-created_at'],
            },
        ),
    ]



import io
import random
import string
import base64
from .setting import Base
from PIL import Image, ImageDraw, ImageFont

padding_x = 7
colors = ["#9C27B0", "#E91E63", "#F44336", "#2196F3",
          "#3F51B5", "#3F51B5", "#3F51B5", "#00BCD4",
          "#03A9F4", "#CDDC39", "#8BC34A", "#8BC34A",
          "#795548", "#607D8B", "#000000", "#9E9E9E"]


def getColor():
    return colors[int(random.random()*len(colors))]


def getHeight():
    deff = int(Base.image_height-Base.font_size)
    return int(random.random()*deff)


def getWeight():
    return int((Base.image_weight-padding_x)/Base.size)


def randomString(size=6, set_int=True, set_str=True):
    text = ""
    if set_int:
        text += string.digits
    if set_str:
        text += string.ascii_lowercase

    if len(text) == 0:
        text += string.ascii_lowercase

    return ''.join(random.sample(text, size))


def generateImage(text):
    background_color = Base.background_color
    if background_color == "random":
        background_color = "#fff"

    font = ImageFont.truetype('arial.ttf', Base.font_size)
    image = Image.new(
        mode="RGB", size=(Base.image_weight, Base.image_height), color=background_color
    )
    draw = ImageDraw.Draw(image)
    x_position = padding_x
    for item in text:
        text_color = Base.text_color
        if Base.text_color == "random":
            text_color = getColor()
        draw.text((x_position, getHeight()), item, font=font, fill=text_color)
        x_position += getWeight()

    in_mem_file = io.BytesIO()
    image.save(in_mem_file, format="PNG")
    in_mem_file.seek(0)
    img_bytes = in_mem_file.read()

    base64_encoded_result_bytes = base64.b64encode(img_bytes)
    base64_encoded_result_str = base64_encoded_result_bytes.decode('ascii')

    return "data:image/png;base64,"+base64_encoded_result_str


def getsize(font, text):
    if hasattr(font, 'getoffset'):
        return tuple(
            [x + y for x, y in zip(font.getsize(text), font.getoffset(text))])
    else:
        return font.getsize(text)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    version='1.1.0',
    author='redj_ai',
    name='Redj Captcha',
    url='https://redj.ai/',
    description='Redj Captcha',
    author_email='info@redj.ai',
    long_description=long_description,
    install_requires=['Pillow==9.0.0', 'django'],
    long_description_content_type="text/markdown",
    project_urls={
        "Bug Tracker": "https://redj.ai/",
    },
    classifiers=[
        'Framework :: Django',
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License'
    ],
    python_requires=">=3",
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
)
